/*
main.c
.   __________________________________
.  | _______________________________  |
.  | |    ________________          | |
.  | |   |   ___________  |         | |
.  | |    \  \          |_|         | |
.  | |     \  \                     | |
.  | |      \  \                    | |
.  | |       \  |   Parser          | |
.  | |	    /  /                    | |
.  | |    /  /          _           | |
.  | |   /  /__________| |          | |
.  | |   |_______________|          | |
.  | |  sigmacalc.sourceforge.net   | |
.  | |______________________________| |
.  |__________________________________|
.              /        \
.	          /          \

this file is part of Sigma Calculator
Copyleft (C)2012 SigmaCalc
:~)
http://sigmacalc.sourceforge.net

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <windows.h>
#include "../../calc.h"
#include "resource.h"

static char g_szClassName[] = "sigmacalcclass";
static HINSTANCE g_hInst = NULL;
#define IDC_FORMULA_TEXT 1001
#define IDC_HISTORY_LIST 1002
#define IDC_CALCULATE_BUTTON 1003
#define IDC_HISTORY_GROUP 1004
#define IDC_CALCULATE_GROUP 1005
#define IDC_ANSWER_TEXT 1006

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_CREATE:
		CreateWindow("BUTTON", "History",
			WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
			5, 5, 480, 130,
			hwnd, (HMENU)IDC_HISTORY_GROUP, g_hInst, NULL);

		CreateWindow("LISTBOX", "",
			WS_CHILD|WS_VISIBLE|WS_BORDER|WS_VSCROLL|WS_HSCROLL|LBS_NOTIFY ,
			10, 25, 460, 100,
			hwnd, (HMENU)IDC_HISTORY_LIST, g_hInst, NULL);

		CreateWindow("EDIT", "",
			WS_CHILD | WS_VISIBLE |  ES_NOHIDESEL ,
			10, 155, 300, 20,
			hwnd, (HMENU)IDC_FORMULA_TEXT, g_hInst, NULL);

		CreateWindow("BUTTON", "Calculate",
			WS_CHILD|WS_VISIBLE| BS_DEFPUSHBUTTON ,
			320, 155, 150, 20,
			hwnd, (HMENU)IDC_CALCULATE_BUTTON, g_hInst, NULL);

		CreateWindow("EDIT", "",
			WS_CHILD | WS_VISIBLE | ES_READONLY ,
			10, 185, 470, 20,
			hwnd, (HMENU)IDC_ANSWER_TEXT, g_hInst, NULL);


		CreateWindow("BUTTON", "Calculate",
			WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
			5, 135, 480, 80,
			hwnd, (HMENU)IDC_CALCULATE_GROUP, g_hInst, NULL);

		SendDlgItemMessage(hwnd, IDC_FORMULA_TEXT, WM_SETFONT,
			(WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));
		//reset the calculator
		scReset();
		break;
	case WM_SETFOCUS:
		SetFocus(GetDlgItem(hwnd, IDC_FORMULA_TEXT));
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_HISTORY_LIST:
			{
				if(HIWORD (wParam) == LBN_SELCHANGE)
				{
					int index,length;
					char* formula;
					index  = SendMessage (GetDlgItem(hwnd, IDC_HISTORY_LIST), LB_GETCURSEL, 0, 0) ;
					length=SendMessage (GetDlgItem(hwnd, IDC_HISTORY_LIST), LB_GETTEXTLEN, index, 0) + 1 ;
					formula=malloc(length);
					SendMessage (GetDlgItem(hwnd, IDC_HISTORY_LIST), LB_GETTEXT, index, (LPARAM) formula) ;
					//set the edit control string
					SetWindowText(GetDlgItem(hwnd, IDC_FORMULA_TEXT),formula);
					free(formula);

				}
			}
			break;
		case IDC_CALCULATE_BUTTON:
			{
				HWND hFormulaEdit;
				DWORD dwTextLength;
				hFormulaEdit=GetDlgItem(hwnd, IDC_FORMULA_TEXT);
				dwTextLength=GetWindowTextLength(hFormulaEdit);
				if(dwTextLength > 0)
				{
					char* pszText;
					pszText = (LPSTR)GlobalAlloc(GPTR, dwTextLength + 1);
					if(pszText != NULL)
					{
						GetWindowText(hFormulaEdit, pszText, dwTextLength + 1);
						scSetFormula(pszText);
						if(scError.ErrorId == SC_NOERROR )
						{
							scParse();
						}
						if(scError.ErrorId != SC_NOERROR )
						{
							char Error[500];
							sprintf(Error,"Error : %s at position %d \n\n",scErrors[scError.ErrorId-1],scError.ErrorPosition+1);
							SetWindowText(GetDlgItem(hwnd, IDC_ANSWER_TEXT),Error);
							SetFocus(GetDlgItem(hwnd, IDC_FORMULA_TEXT));
							SendMessage (hFormulaEdit, EM_SETSEL, scError.ErrorPosition, scError.ErrorPosition) ;

						}
						else
						{
							double d;
							char result[100];
							d=scEvaluate();
							sprintf(result,"%.15lf",d);
							SetWindowText(GetDlgItem(hwnd, IDC_ANSWER_TEXT),result);
							//SetWindowText(GetDlgItem(hwnd, IDC_FORMULA_TEXT),"");
							SendMessage (GetDlgItem(hwnd, IDC_HISTORY_LIST), LB_ADDSTRING, 0, (LPARAM) pszText) ;

						}
						GlobalFree(pszText);
					}
					else
						SetWindowText(GetDlgItem(hwnd, IDC_ANSWER_TEXT),"Error allocating space!");
				}
				else
				{
					SetWindowText(GetDlgItem(hwnd, IDC_ANSWER_TEXT),"Please enter formula.");
				}
				scClear();
				break;
			}
		case CM_FILE_EXIT:
			PostMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		case CM_HELP_WEBSITE:
			ShellExecute(NULL, "open", "http://sigmacalc.sourceforge.net/", NULL, NULL, SW_SHOWNORMAL);
			break;
		case CM_HELP_ABOUT:
			MessageBox (NULL, "Sigma Calculator sample\nCopyleft Sigmacalc 2012\nvisit http://sigmacalc.sourceforge.net for more information." , "About...", 0);
		}
		break;
	case WM_CLOSE:
		DestroyWindow(hwnd);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hwnd, Message, wParam, lParam);
	}
	return 0;
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX WndClass;
	HWND hwnd;
	MSG Msg;

	g_hInst = hInstance;

	WndClass.cbSize        = sizeof(WNDCLASSEX);
	WndClass.style         = 0;
	WndClass.lpfnWndProc   = WndProc;
	WndClass.cbClsExtra    = 0;
	WndClass.cbWndExtra    = 0;
	WndClass.hInstance     = g_hInst;
	WndClass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
	WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW);
	WndClass.lpszMenuName  = "MAINMENU";
	WndClass.lpszClassName = g_szClassName;
	WndClass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

	if(!RegisterClassEx(&WndClass))
	{
		MessageBox(0, "Window Registration Failed!", "Error!",
			MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
		return 0;
	}

	hwnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
		g_szClassName,
		"Sigma Calculator",
		WS_DLGFRAME | WS_SYSMENU ,
		CW_USEDEFAULT, CW_USEDEFAULT, 500, 280,
		NULL, NULL, g_hInst, NULL);

	if(hwnd == NULL)
	{
		MessageBox(0, "Window Creation Failed!", "Error!",
			MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
		return 0;
	}

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	while(GetMessage(&Msg, NULL, 0, 0))
	{
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}
	return Msg.wParam;
}


